/*
 *  TODO
 *     - ensure that the Epsgram test is actually run (it must currently be on the correct page)
 *     - add test for city finder to the Epsgram test
 *
 *     - 'Test parameters preserved' timeout
 *          .inform-dialogue: The chart could not be produced (504 Gateway timeout)
 *             we should:
 *               1- ensure that the test does not finish before the gateway timeout message is delivered (timeout is 30s on stage)
 *               2- check that 504 is captured by the test, and possibly also to either replace inform-dialogue with notify, or check for .inform-dialogue in the test.
 */
import $ from 'jquery'
import Bus from 'bus'
import Unit from 'unit'
import Utils from 'utils'
import Tests from 'test-lib'
import Testable from './testable'
import GridTestable from 'grid/testable'
import router from 'chart-app/router'
import qs from 'page/query'
import Products from 'grid/products'
import User from 'user_info'

var dont_run = [
	/*
	'Epsgrams',
	'Thumbnail selection',
	'Facets',
	'Test search',
	'Test results menu',
	'Test parameter menu',
	'Test parameters preserved',
	'Test text',
	'Test cache',
	'Test meteogram overlay',
	'Test animation'
	*/
]

new Tests([
	{
		name: 'Test login',
		start: function(){
			this.assert_no_element('a[href="/sso/login/openid-connect"]').finished()
		}
	},
	/* TODO this test requires the ability to change package
	{
		name: 'Test interactive charts',
		start: function(){
			var test = Testable.test = this;
			Testable.ensure_interactive_product(function() {
				test.finished();
			});
		}
	},
	*/

	/*   TODO this test fails because transition to 'web' package is not complete
	 *        It will work if 'web' is the initial package
	 */
	{
		name: 'Epsgrams',
		start: function(){
			var test = Testable.test = this;
			if(!Testable.is_epsgram_page()) return test.finished();

			Testable.ensure_epsgram_page(function() {
				Testable.check_recently_viewed(function() {
					Testable.check_location_dialogue().then(function(){
						test.finished()
					})
				})
			})
		}
	},
	{
		// The thumbnail for the current chart should be selected and should be within the visible scrollwindow.
		name: 'Thumbnail selection',
		start: function(){
			var test = Testable.test = this
			test.set_timeout(40000)

			Products.have.filtering.done(function() {
				// note that with pseries, each of these functions is run imediately
				test.pseries([
					Testable.ensure_product_page,
					test.wait_for3('section.results .ps-scrollbar-y', 'results scrollbar (please check results are not filtered)'),
					test.wait_for3('section.results .selected2', 'results selection'),
					Testable.ensure_landing_page,
					Testable.select_product_from_main_list(-1),
					Testable.select_product_from_main_list(0),
					Testable.select_product_from_main_list(1),
					Testable.select_product_from_main_list(Math.round(Products.filtered.length / 2))
				])
			})
		}
	},
	{
		name: 'Facets',
		start: function(){
			var test = Testable.test = this;
			var facets;
			var min_products = 3;

			Testable.ensure_landing_page().then(function(){
				var _g = '.column-2 .catalogue-grid'
				test.assert(_g)
				var g = new GridTestable(test, _g);
				g.ready.then(function(){
					function wait_for_scrollbar(done){
						if(false && !~$.inArray(router._path.dir, ['medium_range', 'mofc_multi'])){
							// scrollbar will be present for packages with many products
							test.wait_for(g.bind(g.has_scrollbar), 'has scrollbar').then(done);
						}else{
							setTimeout(done, 1000);
						}
					}

					test.series([
						function(done){
							test.assert($('.column-1 > .filters h2:first').text() === 'Filters', 'h2: ' + $('.column-1 > .filters h2:first').text());
							test.wait_for('.column-1 .facets', 'facets block').then(function(){
								facets = $('.column-1 .facets');
								facets.find('input[type="checkbox"]:checked').click();
								test.assert(facets.find('input[type="checkbox"]:checked').length === 0, 'checkboxes not cleared');

								test.wait_for(g.bind(g.has_status), 'has status').then(function(){
									console.log('n_products=', g.status_count());
									test.assert(g.status_count() >= min_products, 'not enough products: ' + g.status_count());
									test.wait_for(function(){ return g.el.height() > 80 }, 'height').then(function(){
										done();
									});
								});
							});
						},
						function(done){
							test.assert(g.el.height() > 80, 'grid height:' + g.el.height() + ' el=' + g.el.attr('class'));
							test.wait_for(g.bind(g.has_thumbnails), 'has thumbnails').then(function(){
								test.wait_for(
									function(){ return g.el.find('.grid-items .thumbnail-wrapper').length >= min_products},
									function(){ return 'not enough thumbnails: ' + g.el.find('.grid-items .thumbnail-wrapper:visible').length})
									.then(function() {
										test.assert(g.el.find('.grid-items .thumbnail-wrapper:visible').length >= min_products, 'not enough visible thumbnails: ' + g.el.find('.grid-items .thumbnail-wrapper:visible').length);
										wait_for_scrollbar(function(){
											select_facet(function(){
												if(!~$.inArray(router._path.dir, ['s2s', 'medium'])){ // first facet of s2s brings 24 products which is not enough to reliably remove the scrollbar
													test.wait_for(g.bind(g.not_has_scrollbar), 'scrollbar hide')
														.then(done);
													}else{
														done()
													}
											});
										});
								});
							});

							function select_facet(done) {
								var facet = facets.find('li li:first');
								var n_facet_products = parseInt(facet.text().replace(/.*\((.*)\)/, '$1'), 10);
								test.assert(n_facet_products > 0, 'no facet counts');
								facet.find('input').click();
								test.wait_for(function(){ return g.status_count() === n_facet_products; }, 'n products=' + g.status_count() + ' expected=' + n_facet_products)
									.then(done);
							}
						},
						function clear_facets(done){
							Testable.clear_facets(done);
						}
					], function(){ console.log('finished'); });
				});
			});
		}
	},
	{
		name: 'Test search',
		start: function(){
			var test = Testable.test = this;
			var g = new GridTestable(test, '.column-2 .catalogue-grid');
			g.ready.then(function(){
				test.series([
					function(done) {
						var names = {'chart-catalogue': 'w_t2m30mw_acmad'}
						var product_name = names[router._path.dir] || g.first_product().name();
						$('.thumbnail-list-filter input').val(product_name).trigger('keyup');
						test.wait_for(
							function(){	return g.n_visible_items() === 1 },
							function(){ console.log('visible_items:', g.visible_items()); return 'show single search result: ' + product_name + ' n=' + g.n_visible_items()})
						.then(function(){
							$('.thumbnail-list-filter input').val('').trigger('keyup');
							test.wait_for(
								function(){	return g.n_visible_items() > 1 },
								function(){ return 'clear search: ' +  product_name + ' n_visible=' + g.n_visible_items()})
							.then(done)
						})
					}
				]);
			})
		}
	},
	/* The results menu has been removed
	{
		name: 'Test results menu',
		start: function(){
			var test = Testable.test = this;
			test.set_timeout(10000);

			test.series([
				function(done) {
					Testable.select_product_via_menu(1, done);
				}
			], function(){ console.log('finished'); });
		}
	},
	*/
	/* TODO fix the ability to change package
	{
		name: 'Tropical cyclones',
		start: function() {
			var test = this;
			test.series([
				function(done){
					router.set_dir('tcyclone');
					done();
				}
			], function(){ console.log('finished'); });
		}
	}
	*/
	{
		name: 'Test parameter menu',
		start: function() {
			var test = Testable.test = this

			test.pseries([
				function(){ return test.sleep(2000) },
				Testable.ensure_product_page,
				function(){ return test.set_timeout(60000) }, // multiple chart requests
				Testable.change_parameters.bind(Testable),
			])
		}
	},
	{
		/*
		 *  Test that parameters set for a chart product are preserved when the product is changed.
		 *
		 *  TODO test parameters other than base time.
		 */
		name: 'Test parameters preserved',
		start: function() {
			var test = Testable.test = this

			test.series([
				// change base-time and check it is preserved
				function(done){ Testable.ensure_product_page(done) },
				function(done){ Testable.reset_parameters().then(done) },
				function(done){ test.set_timeout(16000); done() },
				function(done){ Testable.change_base_time(2, done) },
				function(done){ test.set_timeout(16000); done() },
				function(done){ Testable.change_base_time(3, done) }/*,

				// TODO for this to work reliably, we need to pick products that have common parameter values
				function(done){ Testable.change_base_time(3, function(){
					if($('.www-chart-container .basic').length){
						var chart = $('.www-chart-container .basic').basicChart('instance');
						var base_time = chart.product.variables.time.value.base_time;

						Testable.select_product_via_menu(1, function(){
							console.log('checking basic chart...');
							var new_base_time = chart.product.variables.time.value.base_time;
							test.assert(new_base_time.getTime() === base_time.getTime(), 'base time equal: ' + base_time + ' ' + new_base_time);
							done();
						});
					}else{
						console.error('FIXME unhandled chart type');
					}
				})}
				*/
			], function(){ console.log('finished') })
		}
	},
	{
		name: 'Test text',
		start: function(){
			var test = Testable.test = this;
			test.series([
				function(done){
					Testable.ensure_product_page(function(){
						test.wait_for(function() {
							return $('.chart-column .product-text').length === 1;
						}, 'product text').then(function(){
							test.wait_for(function() {
								return $('.chart-column .product-text').text().length > 20;
							}, 'product text content').then(done)
						});
					});
				}
			]);
		}
	},
	{
		// test that repeated views of the same product are successful.
		name: 'Test cache',
		start: function(){
			var test = Testable.test = this;

			var tests = [
				Testable.clear_facets
			]

			// cycle through several products:
			var n_products = 3
			var n_iterations = 10
			Array.apply(0, Array(n_products * n_iterations)).map(function(_, i) {
				tests.push(function(done) { test.set_timeout(10000); Testable.set_product(i % n_products, done); });
			})

			// check spinner. there can be multiple spinners temporarily while products are changed quickly.
			tests.push(function(done) {
				var chart = $('.basic');
				test.wait_for(function() {
					return chart.find('.spinner').length < 2;
				}, 'single spinner').then(done);
			});

			test.series(tests, function(){ console.log('finished'); });
		}
	},
	{
		name: 'Test meteogram overlay',
		start: function() {
			var test = Testable.test = this;

			switch(router._path.dir){
				case 's2s': // no clickable products, but test needs to pass on cams website
					return test.finished();
			}

			function click(el) {
				var position = el.offset()
				console.log('click: position=', position)
				var x = position.left + el.width() / 2;
				var y = position.top + el.height() / 2;

				var ev = new MouseEvent('click', {
					'view': window,
					'bubbles': true,
					'cancelable': true,
					'screenX': x,
					'screenY': y
				})

				el[0].dispatchEvent(ev);
			}

			// global chart has areas which are not clickable
			function ensure_not_global(){
				return new Promise(function(resolve){
					var proj = $('#projection-navigator .ui-button')
					if(~proj.text().indexOf('Global')){
						var chart = $('.chart').chart('instance')
						chart.product.set_parameter('projection', 'classical_europe')
						setTimeout(resolve, 4000) // TODO
					}else{
						resolve()
					}
				})
			}

			test.series([
				function(done){
					Testable.ensure_clickable_product(function() {
						if(!$('#chart-legends:visible').length) return done(); // non-interactive chart - no meteogram.

						ensure_not_global()
						.then(function(){ click($('.chart-image')) })
						.then(test.wait_for3(
							function() { return $('.modal:visible').length;	},
							'overlay open'
						))
						.then(function() {
							test.wait_for(
								function() { return $('input.lat, input.lon').length === 2	},
								'lat lon inputs: expected 2, got ' + $('input.lat, input.lon').length
							)
							.then(test.wait_for3(
								function(){
									var errmsg = $('#right-overlay').text()
									test.assert(!~errmsg.indexOf('meteogram can not be produced'), '"meteogram can not be produced" error message is shown')
									var img = $('#right-overlay img')
									return img.length && (img.attr('src').indexOf('stream.ecmwf.int') > 0 || img.attr('src').indexOf('stream-test.ecmwf.int') > 0)
								},
								function(){ console.warn('image=', $('#right-overlay img')); return 'meteogram image' }
							))
							.then(function(){
								$('#location-overlay .close').click();
								test.wait_for(function() {
									return $('#location-overlay:visible').length === 0;
								}, 'meteogram close').then(done);
							})
						})
					})
				}
			], function(){ console.log('test_meteogram finished') })
		}
	},
	{
		name: 'Test animation',
		start: function() {
			var test = Testable.test = this
			test.series([
				function(done){
					Testable.ensure_animatable_product(function(){
						Testable.test_animation(done)
					})
				}
			], function(){ console.log('finished') })
		}
	}
]).dont_run(dont_run).run()


function LocationBoxTestable(test) {
	this.test = test
	this.el = $('#location-select')
}

LocationBoxTestable.prototype = {
	inputs: function() {
		this.test.assert(this.el, 'inputs: el: ' + this.el.length)
		this.test.assert(this.el.find('#cities input:visible'), 'inputs: city ' + this.el.find('#cities input:visible').length + ' ' + this.el.find('#cities').length)
		return {
			lat: this.el.find('input.lat'),
			lon: this.el.find('input.lon'),
			city: this.el.find('#cities input:visible')
		}
	},

	set_lat: function(lat) {
		var self = this, test = this.test
		test.set_timeout(10000);

		return new Promise(function(resolve){
			self.ensure_open().then(function() {
				test.assert('input.lat')
				var input = self.el.find('input.lat')
				if(Unit.parseLL(input.val()) === Unit.parseLL(lat)) return resolve()
				console.log('set_lat: oldval=', Unit.parseLL(input.val()), 'newval', Unit.parseLL(lat));

				input.val(lat).trigger('change')
				test.assert(input.val() === '' + lat, 'new lat val: ' + input.val())
				self.submit().then(function(product) {
					var lat2 = product.variables.lat.value
					test.assert(lat2 === '' + Unit.parseLL(lat), 'lat value:' + lat2 + ' expected:' + Unit.parseLL(lat))
					test.wait_for(
						function(){ return qs.parse(location.search.slice(1)).lat === '' + Unit.parseLL(lat) },
						function(){ return 'qparam: ' + qs.parse(location.search.slice(1)).lat})
					.then(function() {
						resolve()
					})
				})
			})
		})
	},

	set_lon: function(lon) {
		var self = this, test = this.test
		test.set_timeout(10000);

		return new Promise(function(resolve){
			self.ensure_open().then(function() {
				test.assert('input.lon', 'lon: ' + $('input.lon:visible').length)
				var input = self.el.find('input.lon')
				if(input.val() === '' + Unit.parseLL(lon)) return resolve()

				input.val(lon).trigger('change')
				test.assert(input.val() === '' + lon, 'new lon val: ' + input.val())
				self.submit().then(function(product) {
					var lon2 = product.variables.lon.value
					test.assert(lon2 === '' + Unit.parseLL(lon), 'lon value:' + lon2 + ' expected:' + Unit.parseLL(lon))
					test.wait_for(
						function(){ return qs.parse(location.search.slice(1)).lon === '' + Unit.parseLL(lon) },
						function(){ return 'qparam' + qs.parse(location.search.slice(1)).lat})
					.then(function() {
						resolve()
					})
				})
			})
		})
	},

	check_city: function() {
		var self = this, test = this.test

		return new Promise(function(resolve){
			self.ensure_open().then(function() {
				var selector = '#cities input:visible'
				var input = self.inputs().city
				test.assert(input, 'cities: ' + $(selector).length)
				input.val('').trigger('change')
				setTimeout(function() {
					input.val('Reading')
					send_key_event(input[0])
					test.wait_for('body > .autocomplete-suggestions').then(function() {
						test.wait_for('body > .autocomplete-suggestions .autocomplete-suggestion:first').then(function() {
							var item = $('body > .autocomplete-suggestions > div:contains("Jamaica")')
							test.assert(item, 'Jamaica')
							test.wait_for(
								function(){ return input.val() === 'Reading, Jamaica' },
								function(){ return 'check_city: selection ' + self.inputs().city.val() }
							).then(function(){
								test.wait_for(
									function(){ return self.inputs().lat.val() === '18.45 N' },
									function(){ return 'check_city: lat' +  self.inputs().lat.val() }
								).then(function() {
									self.submit().then(function(product) {
										test.wait_for(
											function(){ return Math.abs(product.variables.lat.value - 18.45) < 0.25 },
											function(){ return 'check_city: variable ' + product.variables.lat.value + ' expected 18.45'}
										).then(function() {
											resolve()
										})
									})
								})
							})
							Test.mousedown(item)
							input.trigger('change')
						})
					})
				}, 1000)
			})
		})
	},

	// open and close the dialogue and check the values are unchanged
	check_no_change: function() {
		var self = this, test = this.test

		return new Promise(function(resolve){
			var chart = $('#basic-chart')
			chart = chart.basicChart('instance')
			var product = chart.product
			var variables = product.variables
			product.variables.station_name.once(self, function() {
				test.fail('unexpected station name change');
			})
			product.variables.lat.once(self, function() {
				test.fail('unexpected lat change');
			})

			function check_values() {
				var inputs = self.inputs()
				test.assert(inputs.city.val() === product.variables.station_name.value, 'city value: ' + inputs.city.val() + ' expected: ' + product.variables.station_name.value)
				test.assert(Unit.parseLL(inputs.lat.val()) === +product.variables.lat.value, 'lat value: ' + Unit.parseLL(inputs.lat.val()) + ' expected: ' + product.variables.lat.value)
				test.assert(Unit.parseLL(inputs.lon.val()) === +product.variables.lon.value, 'lon value: ' + Unit.parseLL(inputs.lon.val()) + ' expected: ' + product.variables.lon.value)
			}

			var values = {lat: variables.lat.value, city: variables.station_name.value}

			self.ensure_open().then(function() {
				check_values()

				self.submit(true).then(function() {
					setTimeout(function() {
						variables.station_name.unsubscribe(self)
						variables.lat.unsubscribe(self)

						test.assert(variables.station_name.value === values.city, 's')
						test.assert(variables.lat.value === values.lat, 's')

						resolve()
					}, 500) // wait to check variable is unchanged
				})
			})
		})
	},

	ensure_open: function() {
		var self = this, test = this.test

		return new Promise(function(resolve){
			$('#location-select:visible').length || $('#location-navigator').click()
			test.wait_for('#location-select').then(function() {
				test.wait_for('#location-select .form-location').then(function() {
					self.el = $('#location-select')
					test.assert(self.el, 'ensure_open: el')
					var inputs = self.inputs()
					test.assert(inputs.lat, 'ensure_open: lat')
					test.assert(inputs.lon, 'ensure_open: lat')
					test.assert(inputs.city, 'ensure_open: city')
					resolve()
				})
			})
		})
	},

	submit: function(unchanged) {
		var self = this, test = this.test

		return new Promise(function(resolve){
			test.assert('.www-chart-container .chart')
			var chart = $('.www-chart-container .chart')
			var bus = chart.basicChart && chart.basicChart('instance') && chart.basicChart('instance').bus
			function on_update(msg) {
				test.wait_for_remove('#location-select').then(function() {
					resolve(msg && msg.product)
				})
			}

			unchanged || bus.listen_once('chart-content-update', function(_, msg){
				on_update(msg)
			}, null, true)

			self.el.find('.btn.primary').click()

			if(unchanged) on_update()
		})
	}
}

/*
 *   Currently this just serves the purpose of triggering event handlers, it doesnt send a meaningful event.
 */
function send_key_event(el) {
	var e = new KeyboardEvent('keyup', {
		key: 'a'
	})
	el.dispatchEvent(e)
}
