import $ from 'jquery'
import Utils from 'utils'
import Bus from 'bus'
import router from 'chart-app/router'
import Products from 'grid/products'
import GridTestable from 'grid/testable'
import User from 'user_info'
import Browser from 'browser'
import ParameterMenuTestable from 'chart/parametermenu_testable'

var Testable = {
	chart: {
		element: '.www-chart-container, .chart-column .chart' // TODO cannot use this (all charts now have .chart, so this fails with n=2 for basic charts)
	},

	ensure_product_page: function(_done) {
		var test = Testable.test

		return new Promise(function(resolve) {
			function done() {
				resolve()
				_done && _done()
			}

			if(router._path.page){
				return done()
			}

			var g = new GridTestable(test, '.column-2 .catalogue-grid');
			g.ready.then(function(){
				// a variation of Utils.findWhere what assumes that the properties are getter functions instead of plain keys
				function findWhere(list, properties){
					return $.map(list, function(l){
						if($.map(properties, function(v, i){
							if(l[i]() === v) return true
						}).length) return l
					})[0]
				}

				var first_product = Products.items[Products.filtered[0]]
				test.assert($('.column-2 .grid-items div.item:visible').length > 0, 'no grid items')
				var pname = g.first_product().name()
				$('.column-2 .grid-items div.item:visible:first').click()
				if(pname !== first_product.name()) console.warn('why is first thumbnail not the first in the filtered list?', pname, product.name)
				var product = findWhere(Products.items, {name: pname})
				test.set_timeout(45000) // have to set v long timeout because chart request is not prioritised over thumbnails
										// er... but shouldn't the title change imediately?
				test.wait_for(
					function() {
						return $(Testable.chart.element).is(':visible') && $('h2:contains(Results)').length && ($('h1:first').text() === product.title_())
					},
					function() { return 'product page: expected "' + product.title_() + '"  got "' + $('h1:first').text() + '"'}
				).then(done)
			})
		})
	},

	ensure_landing_page: function(_done) {
		var test = Testable.test

		return $.Deferred(function(df) {
			console.log('%censure_landing_page', 'background-color:#ff9')

			function done() {
				df.resolve()
				_done && _done()
			}

			Products.packages.then(function(packages) {
				var pkg = (function get_package(){
					var p = Products.items[Products.filtered[0]];
					return p.data.package;
				})()

				// TODO fix the products
				if(pkg === 'medium'){
					console.warn('product claims to be in nonexistent medium package');
					pkg = 'catalogue'
				}

				var p = Utils.findWhere(packages, {name: pkg})
				test.assert(!!p, 'package not found. pkg=' + pkg)
				function is_package_landing(){
//					return !!~$.inArray($('h1:visible').text(), [p.title, 'Graphical products catalogue']);
return !!~$.inArray($('h1:visible').text(), [p.title, 'Graphical products catalogue', 'Charts']);
				}

				if(is_package_landing()) return done()

				var button = $('section.navigation .ui-button')
				var button_text = button.text()
				test.assert(button.length === 1, 'back button')
				button.click()
				test.wait_for(
					is_package_landing,
					function(){
						console.warn('clicked on button:', button)
						return 'ensure_landing_page: expected package landing page: ' + p.title + ' got: "' + $('h1:visible').text() + '" button: ' + button_text
					})
					.then(done)
			});
		})
	},

	ensure_animatable_product: function(done) {
		var self = this
		Testable.test._set_timeout_info('animatable product')

		function set_product(name, title){
			var basic = $(self.chart.element).find('.basic')
			if(basic.length){
				var bus = basic.messageBus()
				var message = 'product-swapped-and-loaded'
			}else{
				bus = $(self.chart.element).chart('instance').options.messageBus
				message = 'chart-content-update'
			}
			bus.clear().listen_once(message, function(){
				have_product()
			})
			router.set_product(name, title)
		}

		function have_product() {
			Testable.test.wait_for('.time-nav .canvas-slider', 'time navigator').then(done)
		}

		switch(router._path.dir){
			case 'catalogue':
				if(router._path.page !== 'medium-2mt-wind30'){
					set_product('medium-2mt-wind30', '2m temperature and 30m winds')
				}else{
					set_product('medium-mslp-wind850', 'Mean sea level pressure and wind speed at 850 hPa')
				}
				break;
			case 'medium':
				if(router._path.page !== 'w_t2m30mw_acmad'){
					set_product('w_t2m30mw_acmad', '2m temperature and 30m winds (ACMAD)')
				}else{
					set_product('w_mslrain_acmad', 'Rainfall and MSLP over Africa (ACMAD)')
				}
				break;
			case 'medium_range':
				have_product()
				break;
			case 's2s':
				if(router._path.page !== 's2s_anomaly_cawcr_2MTM'){
					set_product('s2s_anomaly_cawcr_2MTM', 'BoM 2m Temperature Anomaly')
				}else{
					set_product('s2s_EFI_cawcr_EFI', 'BoM EFI')
				}
				break;
			case 'c3s_seasonal':
				have_product()
				break;
			default:
				Testable.test.assert(false, 'package ' + router._path.dir + ' not supported by test')
				break;
		}
	},

	ensure_product_with_basetime: function(done) {
		var test = Testable.test
		test._set_timeout_info('product with base time')

		switch(router._path.dir){
			case 'medium':
				Testable.set_product(router._path.page !== 'pincr' ? 'pincr' : '/efi2web_2t', done)
				break
			case 'catalogue':
				Testable.set_product(router._path.page !== 'medium-2mt-wind30' ? 'medium-2mt-wind30' : 'medium-mslp-rain', done)
				break
			case 's2s':
				Testable.set_product(router._path.page !== 's2s_anomaly_cawcr_2MTM' ? 's2s_anomaly_cawcr_2MTM' : 's2s_anomaly_cawcr_RAIN', done)
				break
			default:
				Testable.test.assert(false, 'ensure_product_with_basetime: package ' + router._path.dir + ' not supported')
				break
		}
	},

	// TODO fix changing of package
	ensure_interactive_product: function(done) {
		router.set_path('medium_range');
		done();
	},

	ensure_clickable_product: function(done) {
		var test = Testable.test;
		test._set_timeout_info('clickable product');

		User.info.then(function(user){
			test.assert(user.uid && user.uid.length > 0)
		})

		switch(router._path.dir){
			case 'catalogue':
				Testable.set_product(router._path.page !== 'medium-2mt-wind30' ? 'medium-2mt-wind30' : 'medium-mslp-rain', done);
				break;
			case 'medium_range':
				Testable.set_product(router._path.page !== 'medium-mslp-mean-spread' ? 'medium-mslp-mean-spread' : 'medium-multi-efi', done);
				break;
			case 's2s':
				Testable.test.assert(false, 'ensure_clickable_product: package ' + router._path.dir + ' does not contain clickable products');
				break;
			default:
				Testable.test.assert(false, 'ensure_clickable_product: package ' + router._path.dir + ' not supported');
				break;
		}
	},

	is_epsgram_page: function() {
		return $('h1').text() === 'ENS meteograms'
	},

	ensure_epsgram_page: function(done) {
		var self = this, test = Testable.test;

		if(this.is_epsgram_page()) return done();

		test.app_menu_select('ENS Meteograms').then(function(){
			var link = $('a[href*="classical_meteogram"]:first')
			test.assert(link, 'meteogram link not found')
			link.removeAttr('rel')
			Test.click(link) // this has bubbling enabled
			test.wait_for(
				self.is_epsgram_page,
				'h1: expected "' + 'ENS meteograms' + '"'
			).then(done)
		})
	},

	/*
	 *   Use -1 to select last item
	 */
	select_product_from_main_list: function(product_idx) {
		var self = this, test = Testable.test;

		return function() {
			return $.Deferred(function(df){
				Testable.ensure_landing_page().then(function() {
					var thumbnail = $('.column-2 .grid-items .item:eq(' + product_idx + ')')
					var title = thumbnail.find('.title').text();
					test.assert(!!title.length, 'title');
					thumbnail.click()
					test.wait_for(self.chart.element)
						.then(function(){
							test.wait_for(function(){ return $('h1:first').text() === title }, function(){ return ['Title: expected:', '"' + title + '"', 'got:', '"' + $('h1:first').text() + '"']})
								.then(function() {
									test.wait_for('section.results .selected2', 'results selection ' + product_idx)
									.then(function() {
										var selected = 'section.results .selected2 .col-title'
										test.wait_for(
											// TODO what happens if title ellipsised?
											function(){ console.log('   ', $(selected).text().trim(), $('h1').text().trim(), $(selected).text().trim() === $('h1').text().trim()); return $(selected).text().trim() === $('h1').text().trim() },
											function(){ return 'wrong item selected ' + product_idx + ': "' + $(selected).text().trim() + '" (expected "' + $('h1').text().trim() + '")'}
										).then(function(){
											var scrollable = $('section.results .scrollable')
											test.wait_for(
												function(){
													var top = parseInt($('section.results .grid-items .item:first')[0].style.top)
													var overdraw = 4
													var item_height = 44
													var yref = Math.max(0, scrollable.scrollTop() - overdraw * item_height)
													console.log('    ', $('section.results .grid-items .item:first'), parseInt(top), yref);
													return Math.abs(yref - parseInt($('section.results .grid-items .item:first')[0].style.top)) < 100 },
												function(){ return 'unexpected top: ' + Math.round($('section.results .grid-items .item:first')[0].style.top) + ' (scrolltop=' + scrollable.scrollTop() + ')' }
											)
											.then(df.resolve);
										})
									})
								})
						});
				});
			})
		}
	},

	select_product_via_menu: function(product, done) {
		var self = this, test = Testable.test;

		Testable.ensure_product_page(function(){
			$('span:contains(Filter results)').click();
			test.wait_for('.results-menu .grid-items > div', 'menu open and populated')
				.then(function(){
					var menu = $('.results-menu');
					if(typeof product === 'number'){
						var li = menu.find('.grid-items div.item:visible:eq(' + product + ')');
						if(li.length){
							var title = li.find('.col-title').text();
							test.assert(!!title, 'failed to get title for product:' + product);
							li.click();
							test.wait_for(self.chart.element)
								.then(function(){
									test.wait_for(function(){ return $('h1:first').text() === title }, function(){ return ['Title: expected:', title, 'got:', $('h1:first').text()]})
										.then(done);
								});
						}
					}
				});
		});
	},

	set_product: function(product, done) {
		console.log('%cTestable.set_product', 'background-color:#99f', product);
		var test = Testable.test, self = this;

		Testable.ensure_product_page(function(){
			var chart = $('.chart-column .chart');
			var chart_image
			Testable.test.assert(chart.length !== 0, 'no charts found');
			Testable.test.assert(chart.length < 2, 'wrong number of charts:', chart.length);

			function check_title(){
				return $('h1:first').text() === p.title_();
			}

			// test.wait_for is used 3 times in parallel - to get a meaningful error message, they must all use the same timeout message
			function on_timeout(){
				if(!check_title()){
					return ['set_product: page title: expected:', p.title_(), 'got:', $('h1:first').text()];
				}else{
					return 'img name mismatch: got:<br/>' + chart_image.attr('name') + ', expected:<br/>' + p.name();
				}
			}

			//var img = chart.find('img:first').attr('src');
			switch(typeof product){
				case 'number':
					Testable.test.assert(Products.filtered.length > product, 'not enough products:' + Products.filtered.length);
					var p = Products.items[Products.filtered[product]];

					if(check_title()){
						console.log('Testable.set_product: *** product already set:', product, ': "' + p.title_() + '"');
						return done();
					}
					console.log('%cTestable.set_product: ', 'background-color:#69c', product, [$('h1').text()], ' --> ', [p.title_()]);

					router.set_product(p.name(), p.title_());

					$.when(
						$.Deferred(function(df){
							Testable.test.wait_for(check_title, on_timeout).then(df.resolve);
						}),
						/* the image change test is disabled for now because it is failing sometimes
						$.Deferred(function(df){
							Testable.test.wait_for(function(){ return chart.find('img:first').attr('src') !== img; }, 'image change').then(df.resolve);
						}),
						*/
						$.Deferred(function(df){
							test.wait_for(
								function(){ console.log('  *', $('.chart-column .chart'), $('.chart-column .chart').data('ui-basicChart')); return $('.chart-column .chart').length},
								'chart'
							).then(function() {
								chart = $('.chart-column .chart'); // select again, it changes!
								var bus = chart.basicChart && chart.basicChart('instance') ? chart.basicChart('instance').bus : chart.chart('instance').bus;
								bus.listen_once('chart-content-update', function(){
									chart_image = chart.find('> img:first, .chart-image');
									test.assert(chart_image.length == 1, 'unexpected number of chart images: ' + chart_image.length);
									test.assert($('#time-navigator').length < 2, 'too many time-navigators: ' + $('#time-navigator').length);
									test.wait_for(
										function(){ return !$('.spinner:visible').length || $('.spinner:visible').css('opacity') < 0.0001; },
										'spinner stopped'
									)
									.then(function(){
										test.assert(!$('.fail').length, 'no fail icon');
										test.wait_for(function(){
											return chart_image.css('opacity') > 0.999;
										}, 'chart greyed out').then(function(){
											test.assert(chart_image.attr('name') !== 'undefined', 'img name not set');
											test.wait_for(
												function(){ console.log('       *', chart_image.attr('name'), p.name()); return chart_image.attr('name') === p.name()},
												on_timeout
											).then(df.resolve)
										})
									})
								})
							})
						})
					).then(function(){ console.log('test.set_product: done'); done(); });
					break;
				case 'string':
					test.assert(product.length > 0, 'empty product requested')
					p = Products.product_by_name(product)
					test.assert(!!p, 'cannot find product: ' + product)
					router.set_product(product, p.title_());

					// we do not use a message here to detect when the product has been displayed
					// because when changing from one product type to another, the chart and bus will
					// be destroyed and recreated.
					test.wait_for(
						function(){
							var chart = $(self.chart.element);
							var chart_image = chart.find('> img:first, .chart-image');
							return chart_image.attr('name') === p.name()
						},
						function(){
							var images = chart.find('> img:first, .chart-image')
							if(images.length === 1){
								console.log('image swap: src=', images.attr('src'))
								return 'image swap: expected image with name=' + p.name() + ' got: ' + images.attr('name')
							}else
								return 'image swap: expected image with name=' + p.name() + ' but got: ' + images.length + ' ' + images[0]
						}
					).then(done);
					break;
			}
		});
	},

	set_parameter: function(parameter, value, done) {
		if(typeof value === 'number'){
			true
		}else{
			true
		}
		done()
	},

	open_parameter_menu: function(selector) {
		// the buttons may be destroyed and recreated, so we try it twice
		// -there is still the possibility that the menu may be destroyed imediately after the open test completes?

		var self = this, test = Testable.test

		return new Promise(function(resolve){
			// *** better to use string because object can be removed
			var menu_selector = typeof selector === 'string'
				? selector + ' .parameter-selector'
				: selector.find('.parameter-selector')
			test.assert(
				$(menu_selector).length === 1,
				'open_parameter_menu: menu selector length ' + menu_selector.length
			)

			test.wait_for(
				function(){ return button(true).length === 1 },
				'button'
			)
			.then(function(){
				var uuid = $(menu_selector).chartParameterMenu('instance').uuid
				var t = setTimeout(function(){
					var m = $(menu_selector)
					if(m.length && m.chartParameterMenu('instance') && m.chartParameterMenu('instance').uuid !== uuid)
						console.log('uuid changed - button was recreated?')
					b = button()
					b.click()
				}, 4000)

				var b = button()
				b.click()
				test.wait_for(
					'body > .ui-selectmenu-open li:visible',
					function(){
						console.error('clicked on button:', b);
						if(!b.parents('body').length) console.error('button was removed')
						return 'parameter menu open ' + selector
					}
				)
				.then(function(){
					clearTimeout(t)
					resolve()
				})
			})

			function button(_try){
				var b = typeof selector === 'string'
					? $(selector + ' .ui-button')
					: selector.find('.ui-button')
				if(!_try && !b.length){
					console.log('button: selector=', selector)
					throw Error('button not found')
				}
				return b
			}
		})
	},

	/*
	 *  Make one change to each of the parameter menus
	 */
	change_parameters: function() {
		// TODO put the menus into a known state before starting
		// TODO check that the button text changes
		var self = this, test = Testable.test

		return new Promise(function(resolve){
			var buttons = '.parameter-selector .ui-button:visible'
			test.wait_for(
				function(){ return $(buttons).length > 0},
				'parameter menu button'
			)
			.then(function(){
				var parameter_index = -1
				;(function iter() {
					parameter_index++
					var button = $($(buttons)[parameter_index])
					if(!button.length){
						return resolve()
					}
					var mt = new ParameterMenuTestable(parameter_index)
					mt.test = test
					var menu = button.parent().parent()
					var classlist = menu[0].classList
					self.open_parameter_menu([].slice.call(classlist, 0).map(function(v){ return '.' + v }).join(''))
					.then(function(){
						var index = mt.selected_item()
						test.assert(index > -1, 'menu index')
						mt.select_item(index + 1)
						.then(function(){
							// TODO check the chart is updated
							iter()
						})
					})
				})()
			})
		})
	},

	reset_parameters: function() {
		return new Promise(function(resolve){
			resolve()
		})
	},

	change_base_time: function(value, done) {
		// value can be number, date, or null (null will increment)

		var self = this, test = Testable.test

		function parse_date(str){
			self.test.assert(!!str, 'change_base_time: empty date')

			var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'July', 'Aug', 'Sept', 'Oct', 'Nov', 'Dec']

			var d = str.split(' ')
			if(d.length == 2){
				// seasonal charts, eg 'Dec 2016'
				return new Date(d[1], months.indexOf(month), 0, 0)
			}else{
				var month = d[2].replace(',', '')
				var mm = months.indexOf(month)
				var now = new Date()
				var hour = (+d[3] === now.getFullYear()) ? d[4] : d[3]
				return new Date(now.getFullYear(), mm, d[1], hour)
			}
		}

		// check that an element is receiving click events
		/*
		function button_is_clickable(sel) {
			var button = $(sel);
			var uuid = button.button('instance').uuid;
			test.assert(button.length === 1, 'clickable is single');
			var got_event = false;

			function on_click(){
				if(got_event) return; // workaround for unbinding
				console.log('> got click');
				got_event = true;
				return false
			}
			button.on('click.test', on_click);

			return function() {
				test.assert(button.is(':visible'), 'clickable is visible');
				if(got_event){
					button.off('click.test');
				}else{
					button.click();
				}
				return got_event;
			}
		}
		*/
		test._set_timeout_info('ensuring product...')

		Testable.ensure_product_with_basetime(function(){
			test.wait_for('.parameter-time', 'parameter bar')
				.then(function(){
					test.wait_for('.parameter-time .ui-button:visible', 'base-time menu button').then(function(){
						self.open_parameter_menu('.parameter-time').then(function(){
							Testable.test._set_timeout_info('changing date')

							var menu = 'body > .parameter-menu:first > ul'
							test.wait_for(menu)
							.then(function(){
								var mt = new ParameterMenuTestable(0)
								mt.test = Testable.test
								var v = -1
								var index = mt.selected_item()

								if(value === null){
									if(~index){
										v = index + 1
									}
								} else if(typeof value === 'number'){
									v = value
									test.assert(v !== index, 'value is already selected')
								}else{
									console.warn('changing menu by value not implemented')
								}

								if(!~v) throw Error('change_base_time: no index')
								if(!!~v){
									var item = $('body > .ui-selectmenu-open li:eq(' + v + ')');
									test.assert(item.length === 1, 'failed to get base menu item ' + v)
									var target_basetime = parse_date(item.text())

									new Promise(function(resolve) {
										test._set_timeout_info('waiting for chart-content-update ...')
										if(!$(self.chart.element).find('.basic').length){
											// interactive chart
											$(self.chart.element).messageBus().listen_once('chart-content-update', function(){
												var basetime = parse_date($('.layer-title h4').text())
												if(basetime.getTime() === target_basetime.getTime()){
													resolve()
												}else{
													console.log('******* date mismatch: got=', basetime, ' expected=', target_basetime)
												}
											}, null, true)
										}else{
											// basic chart
											// TODO check correct time for basic charts
											resolve()
										}
									})
									.then(function() {
										return new Promise(function(resolve) {
											test._set_timeout_info('waiting for title ...')
											var basetime = parse_date($('.layer-title h4').text())
											Testable.test.wait_for_remove(
												'.parameter-time .parameter-menu li',
												function(){ console.log('  basetime:', basetime.getTime(), target_basetime.getTime()); return 'menu close or date'}
											)
											.then(resolve)
										})
									})
									.then(function(values) {
										done()
									})

									item.click()
								}
							})
						})
					})
				})
		})
	},

	clear_facets: function(done){
		Testable.ensure_landing_page(function() {
			Testable.test.assert($('.column-2 .filter-status').length === 1, 'cannot find filter status box');
			(function remove_box(){
				var buttons = '.column-2 .filter-status > .ui-button';
				var n_buttons = $(buttons).length;
				if(n_buttons){
					$(buttons + ':first').click();
					Testable.test.wait_for(function(){ return $(buttons).length === n_buttons - 1; }, 'filter click (' + n_buttons + ')').then(remove_box);
				}else{
					done();
				}
			})();
		});
	},

	test_animation: function(done) {
		var test = Testable.test;
		test.wait_for('.button.animation-toggle', 'Play button').then(function(){
			var chart = $(Testable.chart.element);
			var src = chart.find('img:visible').attr('src');
			var h4 = chart.find('h4').text();
			test.assert(!!src, 'img');
			test.assert_no_element('.ec-toolbar .on', 'not on');
			var got_start = false;
			Bus().listen_once('animation-start', function(){
				got_start = true;
			})
			$('.button.animation-toggle').click()
			test.wait_for(function(){ return got_start }).then(function(){
				var got_stop = false;
				Bus().listen_once('animation-stop', function(){
					got_stop = true;
				})
				test.wait_for('.ec-toolbar .on').then(function(){ // animation button should change to 'pause' symbol
					test.wait_for(function(){ return chart.find('img:visible').attr('src') !== src}, 'image change').then(function(){
						if(h4) test.assert(chart.find('h4').text() !== h4, 'date title changed');
						$('.button.animation-toggle').click()
						test.wait_for(function(){ return got_stop }).then(function(){
							test.wait_for(test.not_element('.ec-toolbar .on'), 'off').then(done);
						})
					})
				})
			})
		})
	},

	// for epsgram page
	check_recently_viewed: function(done) {
		// TODO create new history items in case none already exist, or existing ones are too old
		var test = Testable.test

		var type_map = {
			'10 day': '10d',
			'15 day': '15d',
			'15 day with climate': '15d_with_climate',
			'Plumes': 'plume',
			'10 day wavegram': 'wave'
		}

		test.wait_for('section.history').then(function(){
			test.wait_for('section.history li:first').then(function(){
				var history = $('section.history li')

				var i = 1;
				(function next_history_item(){
					if(i++ > 5){
						return done()
					}
					var next = history.eq(i)

					var loc = next.find('> div > div:eq(0)').text()
					var base_time = next.find('> div > div:eq(1)').text().split(' ')[0]
					var epsgram_type = next.find('> div > div:eq(2)').text()

					next.click()
					test.wait_for(
						function(){
							var date = $('.ui-selectmenu-menu:eq(0) li .ui-icon-check').parent().find('span.date').text()
							return date === base_time;
						},
						'parameter menu base-time'
					).then(function(){
						test.wait_for(
							function(){
								var t = $('.ui-selectmenu-menu:eq(1) li .ui-icon-check').parent().text()
								return type_map[t] === epsgram_type;
							},
							'parameter menu meteogram-type'
						).then(function(){
							test.wait_for(
								function(){
									var chart = $('#basic-chart').basicChart('instance')
									var station_name = chart.product.variables.station_name
									station_name = station_name.value
									return station_name === loc
								},
								'parameter menu location'
							).then(next_history_item)
						})
					})
				})()
			})
		})
	},

	check_location_dialogue: function(done) {
		var test = Testable.test

		return new Promise(function(resolve){
			var dialogue = new LocationBoxTestable(test)

			dialogue.check_no_change().then(function() {
				dialogue.set_lat(30).then(function() {
					dialogue.set_lat(35).then(function() {
						dialogue.set_lat('30.0 N').then(function() {
							dialogue.set_lat('30.0 S').then(function() {
								dialogue.set_lon(10).then(function() {
									dialogue.set_lon(15).then(function() {
										dialogue.set_lon('20.0 W').then(function() {
											dialogue.set_lon('20.0 E').then(function() {
												dialogue.check_city().then(function(){
													resolve()
												})
											})
										})
									})
								})
							})
						})
					})
				})
			})
		})
	},

	test_tcyclone_products: function() {
		var test = Testable.test

		return new Promise(function(resolve){
			var section = document.querySelectorAll('.column-tcyclone-results')
			test.assert(section.length === 1, 'products section')
			section = section[0]
			var products = section.querySelector('ul').querySelectorAll('li a')
			test.assert(products.length === 4, 'n products: ' + products.length)

			var i = -1
			;(function next_product() {
				var product = products[++i]
				if(!product)
					return resolve()

				product.click()
				test.wait_for(
					function(){ return document.querySelector('h1').innerHTML.indexOf(product.innerHTML) === 0 },
					function(){ return 'title: ' + product.innerHTML }
				)
				.then(next_product)
			})()
		})
	},

	tcyclone_base_time: function() {
		var self = this, test = Testable.test

		return new Promise(function(resolve){
			var buttons = '.parameter-selector .ui-button:visible'
			test.wait_for(
				function(){ return $(buttons).length > 0},
				'parameter menu button'
			)
			.then(function(){
				var parameter_index = -1
				;(function iter() {
					parameter_index++
					if(parameter_index)
						return resolve() // only doing first menu
					var button = $($(buttons)[parameter_index])
					if(!button.length){
						return resolve()
					}
					var mt = new ParameterMenuTestable(parameter_index)
					mt.test = test

					test._set_timeout_info('tcyclone_base_time iter' + parameter_index)
					var menu = button.parent().parent()
					var classlist = menu[0].classList
					self.open_parameter_menu([].slice.call(classlist, 0).map(function(v){ return '.' + v }).join(''))
					.then(function(){
						var index = mt.selected_item()
						test.assert(index > -1, 'menu index')
						if(index == mt.n_items() - 1) index = -1
						mt.select_item(index + 1)
						.then(function(){
							// TODO check the chart is updated
							iter()
						})
					})
				})()
			})
		})
	}
}

export default Testable
